%% Attached vorteces detection
clear

% Referance scales and flow variables
Re    = 2500;                  % Reynold's Number
Fr    = sqrt(0.01);            % Frode's Number
l_ref = 2*pi;                  % Reference length
H_bar = 5*pi;                  % Domain height
a_0   = 0.1;                   % Forcing parameter
u     = a_0*l_ref;             % Referance velocity
nu    = 1/Re;                  % Kinematic viscosity

% Loading surface elevation and timesteps
[eta0, timestamps] = load_surface_data();

% Square half side lenght
ds = 0.1;

% Cutoff value for veortex inspection
cutoff_lambda = 2;

% Wavelt scale index for inspection (i.e. the index in the scale vector for
% the scale of intereset)
wlt_scale = 1;

% Taylor microscale very close to the surface
[u_rms, taylor_scale] = taylor_length_scale();

% Initializing vortex strength vector
vortex_strength = [];

% Testing variable
vortex_strength_2 = [];

ds_ind = floor(ds*256/(2*pi))*2 + 1;

% Differential area used for integration
dA = (2*pi/255).^2;

% Loading z coordinates
[~,~,~,~,zz] = load_near_surf_velocities(1);

% Important to note that zz is 1 at the suirface and decreas as we go 
% further down in the domain
% Surface mesh
x_range = linspace(0, l_ref, 256);
y_range = linspace(0, l_ref, 256);
[x_2d, y_2d] = meshgrid(x_range, y_range);
[x_ver, z_ver] = meshgrid(x_range, zz);

% Index of horizontal slice for inspection
% insp_ind = 1:length(zz);
insp_ind = 1;
z_ind = 1;

% Initializing vortex strength variables
true_vortex_strength = zeros(1,length(timestamps));
vortex_wlt_strength  = zeros(1,length(timestamps));
vortex_elev_strength = zeros(1,length(timestamps));


%% Velocity field comparison at different timesteps

for data_index = 1:length(timestamps)
% Data index for the timestamp we are looking at

% Loading subsurface velocities (u,v,w) and the presure (p)
[v,u,w,p,zz] = load_near_surf_velocities(data_index);

% Surface elevation at this time index
surf_elev = reshape(eta0(data_index,:,:),[256 256]);


%% Finding the derivatives

% Differentiating the surface elevation
% Time step for differentiation
dt = (timestamps(2) - timestamps(1));

% Grid variables in the x ang y directions
xhi = linspace(0,l_ref,256);
psi = linspace(0,l_ref,256);
dx  = xhi(2) - xhi(1);

% The relevant derivatives
[deta_dx, deta_dy] = gradient(surf_elev, xhi, psi);

% Surface gradient and surface curvature
% Finding the second order derivatives
[deta_dxx, deta_dxy] = gradient(deta_dx, xhi, psi);
[~, deta_dyy] = gradient(deta_dy, xhi, psi);

% Surface gaussian curvature, the product of the principal curvatures
surf_gaus_curv = (deta_dxx.*deta_dyy-deta_dxy.^2)./...
    (1 + deta_dx.^2 + deta_dy.^2).^2;

% Surface mean curvature, the mean of the principal curvatures
surf_mean_curv = ((1+deta_dy.^2).*deta_dxx + (1+deta_dx.^2).*deta_dyy ...
    - 2*deta_dx.*deta_dy.*deta_dxy)./(1 + deta_dx.^2 + deta_dy.^2).^(3/2);

% Inicializing principal curvatures
surf_prin_curv_1 = zeros(256,256);
surf_prin_curv_2 = zeros(256,256);

% Setting the principal curvatures
for i = 1:256
    for j = 1:256
        surf_II_form = [deta_dxx(i,j) deta_dxy(i,j); ...
            deta_dxy(i,j) deta_dyy(i,j)]...
            ./sqrt(1 + deta_dx(i,j)^2 + deta_dy(i,j)^2);
        prin_curv = eig(surf_II_form);
        surf_prin_curv_1(i,j) = max(prin_curv);
        surf_prin_curv_2(i,j) = min(prin_curv);
    end
end

% Mean surface elevation
surf_mean = mean(surf_elev,'all');


%% Finding vortices

% Getting the middel eigenvalue at each point (lambda), the vorticity inn
% all trhee directions at each point (omega) and the sum og eignevalues Q
[lambda, omega, Q] = lambda_omega_Q(u, v, w, zz, insp_ind,...
    surf_elev, deta_dx, deta_dy);

% Checking for votices
vortex_pos = zeros(256,256); % Initializing variable
vortex_omega = zeros(256,256);

% When the second eigenvalue (lambda_2) is negative we have a vortex core
% When it is positive we don't, leaving that at 0
% Bathtub vortices will mainly have vorticity in the z directiions
if length(insp_ind) == 1
    vortex_pos = lambda(:,:,1).*(lambda(:,:,1)<0);
    
    % z-direction vorticity for bathtub vortices
    omega_z = reshape(omega(:,:,1,3),[256 256]);
    vortex_omega(lambda<-cutoff_lambda) = omega_z(lambda<-cutoff_lambda);
else
    vortex_pos = lambda(:,:,z_ind).*(lambda(:,:,z_ind)<0);
    
    % z-direction vorticity for bathtub vortices
    omega_z = reshape(omega(:,:,z_ind,3),[256 256]);
    vortex_omega(lambda(:,:,z_ind)<-cutoff_lambda) = ...
        omega_z(lambda(:,:,z_ind)<-cutoff_lambda);
end

% Calculating enstrophy
enstrophy = sum(omega.^2,4);

% Defining x and y coordinates for som bathtub vortices 
[pos_x_vec, pos_y_vec] = bathtub_pos_vec(data_index);

% Defining some random bathtub vortices locations

true_vortex_strength(data_index) = ...
    sum(enstrophy.*(vortex_pos<-0.9), 'all')*dA;
vortex_elev_strength(data_index) = ...
    sum(surf_elev.*(vortex_pos<-0.9), 'all')*dA;


%% Isotropic Wavelet transformation
% Wavelet name
wname_iso = "mexh";
wname_iso_full = "Mexican hat";

% Scale range for transform
scale_range = logspace(0, log10(2^3), 25);

% Wavelet transformation of the surface
cwt_elev_iso = cwtft2(surf_elev, 'wavelet', wname_iso, ...
    'scales', scale_range);

% Matrix the coefficients at each frequency
wlt_elev_iso = reshape(cwt_elev_iso.cfs(:,:,1,:), ...
    [256 256 length(scale_range)]);
angles_coeff_lev = angle(wlt_elev_iso); % Phase angle of the coefficients

[wlt_iso, wlt_iso_len_scale] = ...
    wavelet_length_scale(scale_range(wlt_scale), surf_elev, wname_iso);

% Vortex position circles from wavelet transformation and vortex strength 
% based on the transform of the elevation
[vortex_wlt_strength(data_index), vor_circles, vor_radii] = ...
    wlt_surface_vortices(wlt_elev_iso(:,:,1), surf_mean_curv);

% Calculating vortex strength based on the transform of the elevation
% vortex_wlt_strength(data_index) = vortex_wavelet_strength(vor_circles,...
%     vor_radii, wlt_elev_iso(:,:,1));

fprintf("Timestep %5.2f done, nr %3d of %d \n",...
    timestamps(data_index), data_index, length(timestamps))

end


%% Ploting

if data_index == 160
    
    figure(1), clf
    plot(timestamps, true_vortex_strength./vortex_wlt_strength)
    xlim([timestamps(1) timestamps(end)])
%     title("Comparison of vortex strength at each timestep")
    
else
    figure(1), clf
    pcolor(x_2d, y_2d, surf_elev)
    hold on
    shading flat
    colorbar
    xlim([0,2*pi])
    ylim([0,2*pi])
    xlabel("x axis")
    ylabel("y axis")
    title("Surface elevation at time = " + timestamps(data_index))
    hold off
    
    figure(2), clf
    pcolor(x_2d, y_2d, wlt_elev_iso(:,:,wlt_scale))
    hold on
    contour(x_2d, y_2d, -vortex_pos*0.5*10^-4, [0.9 0.9]*0.5*10^-4, 'r', ...
        'linewidth', 1)
    shading flat
    colorbar
    xlim([0,2*pi])
    ylim([0,2*pi])
    xlabel("x axis")
    ylabel("y axis")
%     title("Wavelet transform of surface elevation at scale = " ...
%         + scale_range(1))
    hold off
    
    figure(3), clf
%     contourf(x_2d, y_2d, surf_gaus_curv, [0 0]*10^-4)
    contour(x_2d, y_2d, wlt_elev_iso(:,:,wlt_scale).*(surf_gaus_curv>0), ...
        [0.5 0.5]*10^-4,'linewidth', 2)
    hold on
    shading flat
%     contour(x_2d, y_2d, wlt_elev_iso(:,:,wlt_scale).*(surf_gaus_curv>0), ...
%         [0.5 0.5]*10^-4,'linewidth', 2)
%     colorbar
    xlim([0,2*pi])
    ylim([0,2*pi])
    xlabel("x axis")
    ylabel("y axis")
    title("Surface gaussian curvature")
    hold off
    
    figure(4), clf
%     contour_levels = [cutoff_lambda cutoff_lambda];
    contourf(x_2d, y_2d, wlt_elev_iso(:,:,wlt_scale), [0.5 0.5]*10^-4)
%     contour(x_2d, y_2d, -vortex_pos*0.5*10^-4, [0.7 0.7]*0.5*10^-4, ...
%         'linewidth', 2)
    hold on
    contour(x_2d, y_2d, -vortex_pos*0.5*10^-4, [0.9 0.9]*0.5*10^-4, 'r', ...
        'linewidth', 1.5)
    viscircles(vor_circles, vor_radii, "LineWidth", 0.8, "Color", 'b', ...
        "LineStyle", ":");
    shading flat
    % colorbar
    xlim([0,2*pi])
    ylim([0,2*pi])
    xlabel("x axis")
    ylabel("y axis")
%     title("Wavelet transform positive values and attached vortices")
    hold off
    
    figure(5), clf
    pcolor(x_2d, y_2d, surf_mean_curv)
    hold on
    shading flat
    colorbar
    xlim([0,2*pi])
    ylim([0,2*pi])
    xlabel("x axis")
    ylabel("y axis")
    title("Surface mean curvature")
    hold off
    
    
%     figure(6), clf
%     contourf(x_2d, y_2d, wlt_elev_iso(:,:,wlt_scale), [1 1]*10^-4)
%     hold on
%     viscircles(vor_circles,vor_radii);
%     contour(x_2d, y_2d, -vortex_pos*0.5*10^-4, [1.5 1.5]*0.5*10^-4,...
%         'linewidth', 1.5)
%     shading flat
%     % colorbar
%     xlim([0,2*pi])
%     ylim([0,2*pi])
%     xlabel("x axis")
%     ylabel("y axis")
%     title("Attached vortex positions from wavelet transformation")
%     hold off

end


