%% Surface upwelling and downdraghts detection
clear

% Referance scales and flow variables
Re    = 2500;                  % Reynold's Number
Fr    = sqrt(0.01);            % Frode's Number
l_ref = 2*pi;                  % Reference length
H_bar = 5*pi;                  % Domain height
a_0   = 0.1;                   % Forcing parameter
u     = a_0*l_ref;             % Referance velocity
nu    = 1/Re;                  % Kinematic viscosity

% Loading surface elevation and timesteps
[eta0, timestamps] = load_surface_data();

% Cutoff value for veortex inspection
cutoff_lambda = 2;

% Wavelt scale index for inspection (i.e. the index in the scale vector for
% the scale of intereset)
wlt_scale = 1;

% Taylor microscale very close to the surface
[u_rms, taylor_scale] = taylor_length_scale();

% Differential area used for integration
dA = (2*pi/255).^2;

% Loading z coordinates
[~,~,~,~,zz] = load_near_surf_velocities(1);

% Important to note that zz is 1 at the suirface and decreas as we go 
% further down in the domain
% Surface mesh
x_range = linspace(0, l_ref, 256);
y_range = linspace(0, l_ref, 256);
[x_2d, y_2d] = meshgrid(x_range, y_range);
[x_ver, z_ver] = meshgrid(x_range, zz);

% Index of horizontal slice for inspection
% insp_ind = 1:length(zz);
insp_ind = 1;
z_ind = 1;

% Preallocation some variables
scars_loc_corr = zeros(1,length(timestamps));
scars_loc_corr_iso = zeros(1,length(timestamps));

%% Velocity field comparison at different timesteps

for data_index = 5
% Data index for the timestamp we are looking at

% Loading subsurface velocities (u,v,w) and the presure (p)
[v,u,w,p,zz] = load_near_surf_velocities(data_index);

% Surface elevation at this time index
surf_elev = reshape(eta0(data_index,:,:),[256 256]);


%% Finding the derivatives

% Grid variables in the x ang y directions
xhi = linspace(0,l_ref,256);
psi = linspace(0,l_ref,256);

% The relevant derivatives
[deta_dx, deta_dy] = gradient(surf_elev, xhi, psi);

% Calculating velocity derivatives 
[du_dx, ~, ~] = vel_derivatives(u, zz, insp_ind,...
    surf_elev, deta_dx, deta_dy);
[~, dv_dy, ~] = vel_derivatives(v, zz, insp_ind,...
    surf_elev, deta_dx, deta_dy);

% Surface gradient and surface curvature
% Finding the second order derivatives
[deta_dxx, deta_dxy] = gradient(deta_dx, xhi, psi);
[~, deta_dyy] = gradient(deta_dy, xhi, psi);

% Surface gaussian curvature, the product of the principal curvatures
surf_gaus_curv = (deta_dxx.*deta_dyy - deta_dxy.^2)./...
    (1 + deta_dx.^2 + deta_dy.^2).^2;

% Surface mean curvature, the mean of the principal curvatures
surf_mean_curv = 0.5*((1+deta_dy.^2).*deta_dxx + (1+deta_dx.^2).*deta_dyy ...
    - 2*deta_dx.*deta_dy.*deta_dxy)./(1 + deta_dx.^2 + deta_dy.^2).^(3/2);

% Inicializing principal curvatures
surf_prin_curv_1 = zeros(256,256);
surf_prin_curv_2 = zeros(256,256);

% Setting the principal curvatures
for i = 1:256
    for j = 1:256
        surf_II_form = [deta_dxx(i,j) deta_dxy(i,j); ...
            deta_dxy(i,j) deta_dyy(i,j)]...
            ./sqrt(1 + deta_dx(i,j)^2 + deta_dy(i,j)^2);
        prin_curv = eig(surf_II_form);
        surf_prin_curv_1(i,j) = max(prin_curv);
        surf_prin_curv_2(i,j) = min(prin_curv);
    end
end


%% Finding up and downwellings

% Surface divergence
surf_div = du_dx + dv_dy;

% Probable location of up and downwellings
[surf_upwelling, surf_downwelling] = surface_wellings_locations(surf_div);

%% Isotropic Wavelet transformation
% Wavelet name
wname_iso = "mexh";
wname_iso_full = "Mexican hat";

% Scale range for transform
scale_range = logspace(0, log10(2^3), 25);

% Wavelet transformation of the surface
cwt_elev_iso = cwtft2(surf_elev, 'wavelet', wname_iso, ...
    'scales', scale_range);

% Matrix the coefficients at each frequency
wlt_elev_iso = reshape(cwt_elev_iso.cfs(:,:,1,:), ...
    [256 256 length(scale_range)]);


%% Nonisotropic wavelet transformation

wname_mexh_non = {"mexh", {2,4,1}}; % Mexican hat wvaelet 2D
wname_morl_non = {"morl", {6,1,1}}; % Morlet wavelet 2D

% Scale range for transform
scale_range = logspace(0, log10(2^3), 25);

% Angles used in the transform
angles = linspace(-pi/2, pi/2, 13);
% angles = [0 15 30 45 60 75 90 105]/180*pi;

cwt_elev_non = cwtft2(surf_elev, 'wavelet', wname_mexh_non, ...
    'angles', angles, 'scales', scale_range);

wlt_elev_non = reshape(cwt_elev_non.cfs, [256 256 ...
    length(scale_range) length(angles)]);

[wlt_non, wlt_non_len_scale] = ...
    wavelet_length_scale(scale_range(wlt_scale),surf_elev,wname_mexh_non);

wlt_scars = zeros(256, 256);

for i = 1:length(angles)
    wlt_scars = wlt_scars + (real(wlt_elev_non(:,:,wlt_scale,i))<-0.9*10^-4);
end

wlt_scars = wlt_scars.*(surf_mean_curv<0);


%% Correltaion between the up and downwellings lovations

% Anisotropic wavelet
% Location matrix of the wellings found by the wavelet transform
wlt_scars_loc = ones(256).*(wlt_scars>1*10^-4);

% Location matrix of the wellings according to the surface divergense
surf_scars_loc = ones(256).*((surf_upwelling>0.1) + (surf_downwelling>0.1));

% Correlations between the two
scars_loc_corr(data_index) = corr2(wlt_scars_loc,surf_scars_loc);


% Isotropic wavelet
wlt_iso_scars = ones(256).*(wlt_elev_iso(:,:,wlt_scale)<-0.8*10^-4);

% Correlation with the isotropci wavelet
scars_loc_corr_iso(data_index) = corr2(wlt_iso_scars,surf_scars_loc);

fprintf("Timestep %5.2f done, nr %3d of %d \n", ...
    timestamps(data_index), data_index, length(timestamps))


end


%% Plotting

if data_index == 160
    
    figure(1), clf
    plot(scars_loc_corr)
    ylim([0.1 0.7])
%     title("Surface welling location correlation")
    
    figure(2), clf
    plot(scars_loc_corr_iso)
    ylim([0.1 0.7])
%     title("Surface welling location (isotropic wavelet)")
    
else
    
    figure(1), clf
    pcolor(x_2d, y_2d, surf_elev)
    hold on
    shading flat
    contour(x_2d, y_2d, surf_downwelling*10^-4, [0.1 0.1]*10^-4,...
        'g', 'linewidth', 1)
    contour(x_2d, y_2d, surf_upwelling*10^-4, [0.1 0.1]*10^-4,...
        'r', 'linewidth', 1)
%     colorbar
    xlim([0,2*pi])
    ylim([0,2*pi])
    xlabel("x axis")
    ylabel("y axis")
    title("Surface elevation at time = " + timestamps(data_index))
    hold off
    
    figure(2), clf
%     pcolor(x_2d, y_2d, wlt_coeff(:,:,1))
%     contourf(x_2d, y_2d, -wlt_elev_iso(:,:,wlt_scale), [1 1]*10^-4)
%     contourf(x_2d, y_2d, real(-wlt_elev_non(:,:,wlt_scale, wlt_angle)),...
%         [1 1]*10^-4)
    contourf(x_2d, y_2d, wlt_scars, [0.1 0.1]*10^-4)
%     contourf(x_2d, y_2d, -surf_mean_curv, [0.03 0.03])
    hold on
%     contourf(x_2d, y_2d, -wlt_coeff(:,:,1), [1 1]*10^-4)
%     contour(x_2d, y_2d, -surf_mean_curv*10^-4, [0.02 0.02]*10^-4,...
%        'linewidth', 1.5)
    contour(x_2d, y_2d, surf_downwelling*10^-5, [0.1 0.1]*10^-5,...
        'g', 'linewidth', 1)
    contour(x_2d, y_2d, surf_upwelling*10^-5, [0.1 0.1]*10^-5,...
        'r', 'linewidth', 1)
    % contourf(x_2d, y_2d, 10*wlt_coeff(:,:,1), [1 1]*10^-3)
    shading flat
    % colorbar
    xlim([0,2*pi])
    ylim([0,2*pi])
    xlabel("x axis")
    ylabel("y axis")
    % legend("Wavelet transform", "Downwellings", "Upwellings")
%     title("Wavelet transform negative and posisive values comparison")
    hold off
    
    figure(3), clf
    
    subplot(1,2,1)
    pcolor(x_2d, y_2d, surf_scars_loc)
    hold on
    shading flat
    xlim([0,2*pi])
    ylim([0,2*pi])
    xlabel("x axis")
    ylabel("y axis")
    title("Surface wellings according to surface divergence")
    hold off
    
    subplot(1,2,2)
    pcolor(x_2d, y_2d, wlt_scars)
    hold on
    shading flat
    xlim([0,2*pi])
    ylim([0,2*pi])
    xlabel("x axis")
    ylabel("y axis")
    title("Surface wellings fond from wlt tranform")
    hold off
    
    wlt_angle = 7;

    figure(4), clf
    pcolor(x_2d, y_2d, real(wlt_elev_non(:,:,wlt_scale, wlt_angle)))
    hold on
%     contour(x_2d, y_2d, -real(wlt_elev_non(:,:,wlt_scale, wlt_angle)),...
%          [0.9 0.9]*10^-4)
    shading flat
    xlim([0,2*pi])
    ylim([0,2*pi])
    xlabel("x axis")
    ylabel("y axis")
%     title("Anisotropic mexh transform with angle  = " ...
%         + angles(wlt_angle)/pi*180)
    hold off
    
    figure(5), clf
    pcolor(x_2d, y_2d, surf_div)
    hold on
    contour(x_2d, y_2d, surf_downwelling*10, [0.001 0.001]*10,...
        'g', 'linewidth', 1)
    contour(x_2d, y_2d, surf_upwelling*10, [0.001 0.001]*10,...
        'r', 'linewidth', 1)
    shading flat
    colorbar
    xlim([0,2*pi])
    ylim([0,2*pi])
    xlabel("x axis")
    ylabel("y axis")
    title("Surface divergence")
    legend("Surface Divergence", "Downwellings", "Upwellings",...
        "Location", "best")
    hold off
    
    figure(6)
    pcolor(x_2d,y_2d,surf_mean_curv)
    hold on
    shading flat
    colorbar
    xlim([0,2*pi])
    ylim([0,2*pi])
    xlabel("x axis")
    ylabel("y axis")
    hold off
    
%     % Line on the surface at
%     y = 1;
%     y_ind = round(y*256/(2*pi));    
%     figure(6), clf
%     subplot(4,1,1)
%     quiver(reshape(u(:,y_ind,:),[256 59])',reshape(w(:,y_ind,:),[256 59])')
%     xlim([0,256])
%     ylim([45 60])
%     
%     subplot(4,1,2)
%     plot(xhi, surf_elev(y_ind,:))
%     xlim([0,2*pi])
%     
%     subplot(4,1,3)
%     pcolor(surf_elev)
%     hold on
%     shading flat
%     contour(surf_downwelling*10^-2, [0.1 0.1]*10^-2, 'g', 'linewidth', 1)
%     contour(surf_upwelling*10^-2, [0.1 0.1]*10^-2, 'r', 'linewidth', 1)
%     ylim([y_ind-5,y_ind+5])
%     hold off
%     
%     subplot(4,1,4)
%     plot(xhi, surf_div(y_ind,:))
%     xlim([0,2*pi])
%     
%     figure(7), clf
% %     quiver(x_2d, y_2d, u(:,:,1), v(:,:,1))
%     pcolor(x_2d, y_2d, w(:,:,1))
%     hold on
%     quiver(x_2d, y_2d, u(:,:,1), v(:,:,1), 'w')
%     contour(x_2d, y_2d, surf_downwelling*10^-2, [0.1 0.1]*10^-2,...
%         'g', 'linewidth', 1)
%     contour(x_2d, y_2d, surf_upwelling*10^-2, [0.1 0.1]*10^-2,...
%         'r', 'linewidth', 1)
%     shading flat
%     xlim([0,2*pi])
%     ylim([0,2*pi])
%     title("Horisontal velocity at the surface")
%     hold off
    
end




