function [wlt, wlt_length] = wavelet_length_scale(scale, surf_elev, wname)
% Function to fint the size of the wavelet used at each scale. This is
% based on the cwtft2 function in MATLAB. Since cwtft2 uses fft to
% calculated the fourier transfomr it multiplies the fft og the signal with
% the complex conjugate og the fft of the wavelet to calculate the
% convolution needed for the wavelet transform

if iscell(wname)
    name = wname{1};
    var  = [wname{2}{:}];
else
    name = wname;
    var  = [];
end

fimg    = fft2(surf_elev); % fft of the surface elevation
S       = size(fimg);      % Size of the fft      
W       = S(1);            % Width of the fft
% We only use the widht here since our domain is square
W2      = floor((W-1)/2);  % Half width parameter used in cwtft2
W_puls  = 2*pi/W*[ 0:W2  (W2-W+1):-1 ]; % Frequency range in the transform
[xx,yy] = meshgrid(W_puls,W_puls); % Meshgrid of the frequencies
xx      = scale.*xx;       % Scaling of the frequencies
yy      = scale.*yy;       % Scaling of the frequencis

% fft of the wavelet using the scaled frequencies from the fft of the
% original surface elevation
switch name
    case "mexh"
        if isempty(var)
            var = [2,1,1];
            % For the Mexican hat wavelet the first value, here 2, is
            % called the order, it has the effect of adding valleys and
            % tops to the wavelet. I have not found a reason for it yet.
            % The second and third are the stretching if the x and y
            % direction respectively, used for creating asymmetry in the
            % wavelet making it anisotropic
        end
        fft_wlt = scale .* (-2*pi) * (xx.^2 + yy.^2).^(var(1)/2).*...
            exp(-((var(2)*xx).^2 + (var(3)*yy).^2)/ 2);
    case "morl"
        if isempty(var)
            var = [6,1,1];
            % For the non isotropic Morlet wavele the first values is used
            % to adjust the frequency of the sine wave in the gaussina
            % evenlope. The sencond os for stretching the envelope, and the
            % third for stretchin the envelope in just the y axis.
        end
        fft_wlt = exp( - var(2)^2*((xx - var(1)).^2 + (var(3)*yy).^2)/2 );
    otherwise
        error("Not implemented for this wavelet yet")
end

% Inverse fft of the wavelet to give us the actual wavelet
ifft_wlt = ifft2(conj(fft_wlt));

% Preallocating the wavelet
wlt = zeros(W,W);

% Rearragin the wadrants of the inverse to remake the wavelet
wlt(1:W/2,1:W/2) = ifft_wlt(end-W/2+1:end, end-W/2+1:end);
wlt(end-W/2+1:end,1:W/2) = ifft_wlt(1:W/2, end-W/2+1:end);
wlt(1:W/2,end-W/2+1:end) = ifft_wlt(end-W/2+1:end, 1:W/2);
wlt(end-W/2+1:end,end-W/2+1:end) = ifft_wlt(1:W/2, 1:W/2);

% Cut of the whole wavelet going throught the peak of the mexican hat or
% other wavelet
switch name
    case "mexh"
        if var(2)>var(3)
            wlt_cut = wlt(W/2+1:end, W/2+1);
        else
            wlt_cut = wlt(W/2+1, W/2+1:end);
        end
    case "morl"
        wlt_cut = real(wlt(W/2+1, W/2+1:end));
end

% Zooming in at the values around the center of the wavelet
% wlt = wlt(W/2-18:W/2+21,W/2-18:W/2+21);

% For the mexican hat wavelet the next part is to calculte the location of 
% where the wavelet first crosses the x axis. 

for i = 2:W/2-1
    if wlt_cut(i+1)<wlt_cut(i), break, end
end

% Where the wavelet crosses the x-axis
zero_loc = interp1(wlt_cut(1:i), 1:i, 0);

switch name
    case "mexh"
        % Then I used the fact that zero crossin is 1/sqrt(3) parts of the 
        % legnth to the bottom of the valley in the mexican hat. I'm using 
        % the distande to the walley as my wavelet scale. The lengthscale 
        % for the wavelet is then the distance between the two  valleys in 
        % the mexican hat wavelet
        wlt_length = 2*(zero_loc*sqrt(3))/W * (2*pi);
    case "morl"
        % The length scale here is the distance between the two closest
        % valleys in the morlet wavelet, since these are is the smalles
        % scale the wavelet can detect. For the whole wavelet, multiply by
        % about half the frequency (var(1)/2). That shoul give a good
        % estimate for the witdh of the whole wavelet
        wlt_length = 4*(zero_loc)/W * (2*pi);
end

end

